@@ -183,7 +183,6 @@ class PositionBar(NamedTuple):
 class _OrderResult(NamedTuple):
     filled_shares: int
     rem_shares: int
-    pnl: Decimal
 
 
 def _calculate_pnl(
@@ -382,10 +381,10 @@ class Portfolio:
     ) -> _OrderResult:
         pnl = Decimal()
         if symbol not in self.short_positions:
-            return _OrderResult(0, shares, pnl)
+            return _OrderResult(0, shares)
         rem_shares = int(min(shares, math.floor(self.cash / fill_price)))
         if rem_shares <= 0:
-            return _OrderResult(0, shares, pnl)
+            return _OrderResult(0, shares)
         pos = self.short_positions[symbol]
         while pos.entries:
             entry = pos.entries[0]
@@ -398,6 +397,9 @@ class Portfolio:
                 rem_shares -= entry.shares
                 pos.shares -= entry.shares
                 pos.entries.popleft()
+                pnl_per_bar = (
+                    entry_pnl if not entry.bars else entry_pnl / entry.bars
+                )
                 self._add_trade(
                     type=entry.type,
                     symbol=symbol,
@@ -408,15 +410,30 @@ class Portfolio:
                     return_pct=((entry.price / fill_price) - 1) * 100,
                     cumulative_pnl=self.pnl + pnl,
                     bars=entry.bars,
-                    pnl_per_bar=entry_pnl / entry.bars,
+                    pnl_per_bar=pnl_per_bar,
                 )
             else:
                 order_amount = rem_shares * fill_price
                 entry_pnl = (rem_shares * entry.price) - order_amount
-                pnl += (rem_shares * entry.price) - order_amount
+                pnl += entry_pnl
                 self.cash -= order_amount
                 entry.shares -= rem_shares
                 pos.shares -= rem_shares
+                pnl_per_bar = (
+                    entry_pnl if not entry.bars else entry_pnl / entry.bars
+                )
+                self._add_trade(
+                    type=entry.type,
+                    symbol=symbol,
+                    entry_date=entry.date,
+                    exit_date=date,
+                    shares=rem_shares,
+                    pnl=entry_pnl,
+                    return_pct=((entry.price / fill_price) - 1) * 100,
+                    cumulative_pnl=self.pnl + pnl,
+                    bars=entry.bars,
+                    pnl_per_bar=pnl_per_bar,
+                )
                 rem_shares = 0
                 break
         self.pnl += pnl
@@ -424,7 +441,7 @@ class Portfolio:
             del self.short_positions[symbol]
             if symbol not in self.long_positions:
                 self.symbols.remove(symbol)
-        return _OrderResult(shares - rem_shares, rem_shares, pnl)
+        return _OrderResult(shares - rem_shares, rem_shares)
 
     def _buy(
         self,
@@ -519,7 +536,7 @@ class Portfolio:
     ) -> _OrderResult:
         pnl = Decimal()
         if symbol not in self.long_positions:
-            return _OrderResult(0, shares, pnl)
+            return _OrderResult(0, shares)
         rem_shares = shares
         pos = self.long_positions[symbol]
         while pos.entries:
@@ -533,6 +550,9 @@ class Portfolio:
                 rem_shares -= entry.shares
                 pos.shares -= entry.shares
                 pos.entries.popleft()
+                pnl_per_bar = (
+                    entry_pnl if not entry.bars else entry_pnl / entry.bars
+                )
                 self._add_trade(
                     type=entry.type,
                     symbol=symbol,
@@ -543,14 +563,30 @@ class Portfolio:
                     return_pct=((fill_price / entry.price) - 1) * 100,
                     cumulative_pnl=self.pnl + pnl,
                     bars=entry.bars,
-                    pnl_per_bar=entry_pnl / entry.bars,
+                    pnl_per_bar=pnl_per_bar,
                 )
             else:
                 order_amount = rem_shares * fill_price
-                pnl += order_amount - (rem_shares * entry.price)
+                entry_pnl = order_amount - (rem_shares * entry.price)
+                pnl += entry_pnl
                 self.cash += order_amount
                 entry.shares -= rem_shares
                 pos.shares -= rem_shares
+                pnl_per_bar = (
+                    entry_pnl if not entry.bars else entry_pnl / entry.bars
+                )
+                self._add_trade(
+                    type=entry.type,
+                    symbol=symbol,
+                    entry_date=entry.date,
+                    exit_date=date,
+                    shares=rem_shares,
+                    pnl=entry_pnl,
+                    return_pct=((fill_price / entry.price) - 1) * 100,
+                    cumulative_pnl=self.pnl + pnl,
+                    bars=entry.bars,
+                    pnl_per_bar=pnl_per_bar,
+                )
                 rem_shares = 0
                 break
         self.pnl += pnl
@@ -558,7 +594,7 @@ class Portfolio:
             del self.long_positions[symbol]
             if symbol not in self.short_positions:
                 self.symbols.remove(symbol)
-        return _OrderResult(shares - rem_shares, rem_shares, pnl)
+        return _OrderResult(shares - rem_shares, rem_shares)
 
     def _short(
         self,
